# SPDX-License-Identifier: GPL-2.0
#
# Copyright 2019 Google LLC.

import gdb
import zlib

from linux import utils


class LxConfigDump(gdb.Command):
    """Output kernel config to the filename specified as the command
       argument. Equivalent to 'zcat /proc/config.gz > config.txt' on
       a running target"""

    def __init__(self):
        super(LxConfigDump, self).__init__("lx-configdump", gdb.COMMAND_DATA,
                                           gdb.COMPLETE_FILENAME)

    def invoke(self, arg, from_tty):
        if len(arg) == 0:
            filename = "config.txt"
        else:
            filename = arg

        try:
            py_config_ptr = gdb.parse_and_eval("&kernel_config_data")
            py_config_ptr_end = gdb.parse_and_eval("&kernel_config_data_end")
            py_config_size = py_config_ptr_end - py_config_ptr
        except gdb.error as e:
            raise gdb.GdbError("Can't find config, enable CONFIG_IKCONFIG?")

        inf = gdb.inferiors()[0]
        zconfig_buf = utils.read_memoryview(inf, py_config_ptr,
                                            py_config_size).tobytes()

        config_buf = zlib.decompress(zconfig_buf, 16)
        with open(filename, 'wb') as f:
            f.write(config_buf)

        gdb.write("Dumped config to " + filename + "\n")


LxConfigDump()



# SPDX-License-Identifier: GPL-2.0
"""
Hide tristate choice values with mod dependency in y choice.

If tristate choice values depend on symbols set to 'm', they should be
hidden when the choice containing them is changed from 'm' to 'y'
(i.e. exclusive choice).

Related Linux commit: fa64e5f6a35efd5e77d639125d973077ca506074
"""


def test(conf):
    assert conf.oldaskconfig('config', 'y') == 0
    assert conf.config_contains('expected_config')
    assert conf.stdout_contains('expected_stdout')
